%{
Note that, if one sets eps_ct (lambda_trans) = 5; eps_tc (lambda_cis) = 20;
which totals at eps_tot = 25 Hz > nu_ct = 20 Hz, but comparable to the 
flagellar-flow coupling eps_f (scanned from 0:10 Hz for Arnold tongue),
one can see the breakdown of cis-trans synchronization by the flow.
%}
close all
strain = 'ptx1';
Fs = 1000; % [Hz]
dt = 1/Fs; % [s]
t_tot = 100; %[s]
t = 0:1/Fs:t_tot; %[s]

f_c = 45; % [Hz]
f_t = 65; % [Hz]

switch strain
    case 'wt'
        eps_ct = 12; % [Hz]
        eps_tc = 4*eps_ct; % [Hz]       
        T_eff_set = 0.5; % [rad/s] * 2pi
    case 'ptx1'
        eps_ct = 30; % [Hz]
        eps_tc = eps_ct; % [Hz]       
        T_eff_set = 3.0; % [rad/s] * 2pi   3.0 for ptx1
end

f0 = (eps_tc*f_c + eps_ct*f_t)/(eps_tc + eps_ct);
sigma = sqrt(Fs*2*T_eff_set*2*pi);

%% run
eps_f_list =  0:1:10;
f_f_list = 40:75;
N_eps = length(eps_f_list);
N_freq = length(f_f_list);
[xi_ct_mat, xi_cf_mat, xi_tf_mat] = deal(zeros(N_eps,N_freq));

for i_eps = 1:N_eps
    fprintf('epsilon: %d/%d\n',i_eps,N_eps)
    for i_f = 1:N_freq
        eps_f = eps_f_list(i_eps);
        f_f = f_f_list(i_f);
                
        [phi_c, phi_t, phi_f] = deal(zeros(size(t)));

        x_c = rand(size(t));
        zeta_c    = norminv(x_c,0,sigma/sqrt(2));
        x_t = rand(size(t));
        zeta_t    = norminv(x_t,0,sigma/sqrt(2));

        
        phi_c(1) = rand()*2*pi;
        phi_t(1) = rand()*2*pi;
        phi_f(1) = rand()*2*pi;
        for i = 1:length(t)-1
            d_phi_f = 2*pi*f_f*dt;
            phi_f(i+1) = phi_f(i)+ d_phi_f;
            
            d_phi_c = (2*pi*f_c - 2*pi*eps_ct * sin(phi_c(i)-phi_t(i))...
                       - 2*pi*eps_f * sin(phi_c(i)-phi_f(i))+zeta_c(i))*dt;
            phi_c(i+1) =  phi_c(i)+ d_phi_c;
            
            d_phi_t = (2*pi*f_t - 2*pi*eps_tc * sin(phi_t(i)-phi_c(i))...
                       - 2*pi*eps_f * sin(phi_t(i)-phi_f(i))+zeta_t(i))*dt;
            phi_t(i+1) =  phi_t(i)+ d_phi_t;
        end
        
        Delta_ct = phi_c - phi_t;
        Delta_cf = phi_c - phi_f;
        Delta_tf = phi_t - phi_f;
        [~,xi_ct_mat(i_eps,i_f),~] = markSync(Delta_ct/2/pi,Fs);
        [~,xi_cf_mat(i_eps,i_f),~] = markSync(Delta_cf/2/pi,Fs);
        [~,xi_tf_mat(i_eps,i_f),~] = markSync(Delta_tf/2/pi,Fs);
    end
end


%% plot Arnold tongues between three parties
for i_fig = 1:3
    figure(i_fig)
    switch i_fig
        case 1; position = [1,1,6,2];
        case 2; position = [1,4,6,2];
        case 3; position = [1,7,6,2];
    end
    set(gcf,'Units','inches','Position',position,...
        'defaultTextInterpreter','Latex','defaultAxesFontSize',10)
    set(gca,'defaultTextInterpreter','Latex',...
        'tickLabelInterpreter','Latex')
    hold on , box on
    xlabel('$f_{flow}-f_0$ (Hz)')
    ylabel('$\varepsilon$ (Hz)')
    xlim([-12,28]),ylim([-0.5,11])
end

figure(1)
title('Sync between $cis$ and $trans$')
contourf(f_f_list-f0,eps_f_list,xi_ct_mat,50,'LineStyle','none')
colormap(hot)
caxis([-0.05,1.1])
colorbar()
pause(eps)

figure(2)
title('Sync between $cis$ and flow')
contourf(f_f_list-f0,eps_f_list,xi_cf_mat,50,'LineStyle','none')
colormap(hot)
caxis([-0.05,1.1])
colorbar()
pause(eps)

figure(3)
title('Sync between $trans$ and flow')
contourf(f_f_list-f0,eps_f_list,xi_tf_mat,50,'LineStyle','none')
colormap(hot)
caxis([-0.05,1.1])
colorbar()
pause(eps)
